@@ -27,6 +27,7 @@
 
 /**
  * SECTION:tracker-thumbnailer
+ * @title: Thumbnailer management
  * @short_description: Request the thumbnailer service creates or
  * updates thumbnails.
  * @include: libtracker-miner/tracker-miner.h
@@ -62,12 +63,7 @@ typedef struct {
 	gboolean service_is_available;
 } TrackerThumbnailerPrivate;
 
-#if GLIB_CHECK_VERSION (2,31,0)
-static void private_free (gpointer data);
-static GPrivate private_key = G_PRIVATE_INIT (private_free);
-#else
 static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
-#endif
 
 static void
 private_free (gpointer data)
@@ -145,11 +141,9 @@ tracker_thumbnailer_init (void)
 	/* Don't start at 0, start at 1. */
 	private->request_id = 1;
 
-#if GLIB_CHECK_VERSION (2,31,0)
-	g_private_replace (&private_key, private);
-#else
-	g_static_private_set (&private_key, private, private_free);
-#endif
+	g_static_private_set (&private_key,
+	                      private,
+	                      private_free);
 
 	g_message ("Thumbnailer connections being set up...");
 
@@ -275,11 +269,7 @@ error_handler:
 void
 tracker_thumbnailer_shutdown (void)
 {
-#if GLIB_CHECK_VERSION (2,31,0)
-	g_private_replace (&private_key, NULL);
-#else
 	g_static_private_set (&private_key, NULL, NULL);
-#endif
 }
 
 /**
@@ -308,11 +298,7 @@ tracker_thumbnailer_move_add (const gchar *from_uri,
 	g_return_val_if_fail (from_uri != NULL, FALSE);
 	g_return_val_if_fail (to_uri != NULL, FALSE);
 
-#if GLIB_CHECK_VERSION (2,31,0)
-	private = g_private_get (&private_key);
-#else
 	private = g_static_private_get (&private_key);
-#endif
 	g_return_val_if_fail (private != NULL, FALSE);
 
 	if (!private->service_is_available) {
@@ -355,11 +341,7 @@ tracker_thumbnailer_remove_add (const gchar *uri,
 
 	g_return_val_if_fail (uri != NULL, FALSE);
 
-#if GLIB_CHECK_VERSION (2,31,0)
-	private = g_private_get (&private_key);
-#else
 	private = g_static_private_get (&private_key);
-#endif
 	g_return_val_if_fail (private != NULL, FALSE);
 
 	if (!private->service_is_available) {
@@ -394,11 +376,7 @@ tracker_thumbnailer_cleanup (const gchar *uri_prefix)
 
 	g_return_val_if_fail (uri_prefix != NULL, FALSE);
 
-#if GLIB_CHECK_VERSION (2,31,0)
-	private = g_private_get (&private_key);
-#else
 	private = g_static_private_get (&private_key);
-#endif
 	g_return_val_if_fail (private != NULL, FALSE);
 
 	if (!private->service_is_available) {
@@ -436,11 +414,7 @@ tracker_thumbnailer_send (void)
 	TrackerThumbnailerPrivate *private;
 	guint list_len;
 
-#if GLIB_CHECK_VERSION (2,31,0)
-	private = g_private_get (&private_key);
-#else
 	private = g_static_private_get (&private_key);
-#endif
 	g_return_if_fail (private != NULL);
 
 	if (!private->service_is_available) {
